package cz.muni.pa165.car.rest;

import cz.muni.pa165.car.service.CarService;
import cz.muni.pa165.common_library.dtos.CarDto;
import io.swagger.v3.oas.annotations.Operation;
import jakarta.validation.Valid;
import java.util.List;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * Controller for car.
 */
@RestController
@RequestMapping(path = "/car")
@Validated
public class CarController {

  CarService carService;

  /**
   * Constructor for Car Controller.
   *
   * @param carService Car service
   */
  public CarController(CarService carService) {
    this.carService = carService;
  }

  /**
   * Creates a new car.
   *
   * @param carDto the DTO representing the car to be created
   * @return a ResponseEntity containing the DTO of the created car
   */
  @Operation(summary = "Create a car")
  @PostMapping(path = "/", consumes = MediaType.APPLICATION_JSON_VALUE,
      produces = MediaType.APPLICATION_JSON_VALUE)
  public ResponseEntity<CarDto> createCar(@Valid @RequestBody CarDto carDto) {
    return ResponseEntity.ok(carService.postCar(carDto));
  }

  /**
   * Deletes a car by its ID.
   *
   * @param carId the ID of the car to be deleted
   * @return a ResponseEntity containing a message indicating whether the car was deleted or not
   */
  @Operation(summary = "Delete a car")
  @DeleteMapping(path = "/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
  public ResponseEntity<String> deleteCar(@RequestParam Long carId) {
    return ResponseEntity.ok(carService.deleteById(carId));
  }

  /**
   * Retrieves a car by its ID.
   *
   * @param id the ID of the car to retrieve
   * @return a ResponseEntity containing the DTO of the retrieved car
   */
  @Operation(summary = "Get a car")
  @GetMapping(path = "/id={id}", produces = MediaType.APPLICATION_JSON_VALUE)
  public ResponseEntity<CarDto> getCar(@PathVariable("id") Long id) {
    return ResponseEntity.ok(carService.getCarById(id));
  }

  /**
   * Retrieves a list of all cars.
   *
   * @return a ResponseEntity containing a list of DTOs representing all cars
   */
  @Operation(summary = "Get all cars")
  @GetMapping(path = "/", produces = MediaType.APPLICATION_JSON_VALUE)
  public ResponseEntity<List<CarDto>> getAllSeasons() {
    return ResponseEntity.ok(carService.getAllCars());
  }

}
