package cz.muni.pa165.car.service;

import cz.muni.pa165.car.data.model.Car;
import cz.muni.pa165.car.data.repository.CarComponentRepository;
import cz.muni.pa165.car.data.repository.CarRepository;
import cz.muni.pa165.car.data.repository.DriverRepository;
import cz.muni.pa165.common_library.dtos.CarDto;
import cz.muni.pa165.common_library.exception.DatabaseException;
import java.util.ArrayList;
import java.util.List;
import org.springframework.data.util.Pair;
import org.springframework.stereotype.Service;

/**
 * Service for car management.
 */
@Service
public class CarServiceImpl implements CarService {

  CarRepository carRepository;
  CarComponentRepository componentRepository;
  DriverRepository driverRepository;

  /**
   * Constructor for Car Service.
   *
   * @param carRepository       Car repository
   * @param componentRepository Component repository
   * @param driverRepository    Driver repository
   */
  public CarServiceImpl(CarRepository carRepository,
                        CarComponentRepository componentRepository,
                        DriverRepository driverRepository) {
    this.carRepository = carRepository;
    this.componentRepository = componentRepository;
    this.driverRepository = driverRepository;
  }

  @Override
  public CarDto postCar(CarDto carDto) {
    return carConverterToDto(carRepository.save(carDtoConverter(carDto)));
  }

  @Override
  public CarDto getCarById(Long carId) {
    return carConverterToDto(carRepository.findById(carId).orElseThrow(
        () -> new DatabaseException(
            "Finding car with id=" + carId + " was unsuccessful"
        )
    ));
  }

  @Override
  public List<CarDto> getAllCars() {
    return carRepository
        .findAll()
        .stream()
        .map(CarServiceImpl::carConverterToDto)
        .toList();
  }

  @Override
  public String deleteById(Long carId) {
    carRepository.deleteById(carId);
    return "Car with id = " + carId + " deleted!";
  }

  /**
   * Converts Dto object to Car object.
   * TODO: Change to the correct version of converter.
   *
   * @param carDto Dto object of the car.
   * @return Car object.
   */
  public static Car carDtoConverter(CarDto carDto) {

    // TEST VERSION WHEN COMPONENTS AND DRIVERS DO NOT EXIST
    return Car.builder()
        .id(carDto.getId())
        .components(new ArrayList<>())
        .drivers(new ArrayList<>())
        .mainDriver(null)
        .build();

    // TO BE USED IN FINAL VERSION
    /*
    var car = Car.builder()
        .id(carDto.getId())
        .build();

    var components = new ArrayList<CarComponent>();
    for (Pair<Long, String> componentPair : carDto.getComponentIdsNames()) {
      Long id = componentPair.getFirst();
      components.add(componentRepository.findById(id).orElseThrow(
          () -> new DatabaseException("Component not found")
      ));
    }
    car.setComponents(components);
    var drivers = new ArrayList<Driver>();
    for (Pair<Long, String> driverPair : carDto.getDriverIdsNames()) {
      Long id = driverPair.getFirst();
      drivers.add(driverRepository.findById(id).orElseThrow(
          () -> new DatabaseException("Driver not found")
      ));
    }
    car.setDrivers(drivers);
    return car;
    */
  }

  /**
   * Converts Car object to its Dto.
   *
   * @param car Car object.
   * @return Dto object.
   */
  public static CarDto carConverterToDto(Car car) {
    Long id = null;
    if (car.getMainDriver() != null) {
      id = car.getMainDriver().getId();
    }
    return CarDto.builder()
        .id(car.getId())
        .componentIdsNames(car.getComponents()
            .stream()
            .map(carComponent -> Pair.of(carComponent.getId(), carComponent.getName()))
            .toList())
        .driverIdsNames(car.getDrivers()
            .stream()
            .map(driver -> Pair.of(driver.getId(), driver.getName()))
            .toList())
        .mainDriverId(id)
        .build();
  }

}
