package cz.muni.pa165.common_library.validator;

import jakarta.annotation.Nullable;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import javax.validation.ConstraintValidator;
import javax.validation.ConstraintValidatorContext;

/**
 * A custom validator to check if all attributes in a Map of String and Integer pairs
 * contain valid characteristics as defined in the {@link ValidCharacteristics} annotation.
 */
public class CharacteristicsValidator
    implements ConstraintValidator<ValidCharacteristics, Map<String, Integer>> {

  private Set<String> validCharacteristics;

  /**
   * Initializes the validator with the valid characteristics provided in the annotation.
   *
   * @param constraintAnnotation the {@link ValidCharacteristics} annotation
   *                             with the valid characteristics
   */
  @Override
  public void initialize(ValidCharacteristics constraintAnnotation) {
    validCharacteristics = new HashSet<>(Arrays.asList(constraintAnnotation.value()));
  }

  /**
   * Checks if all attributes in the input Map contain valid characteristics.
   *
   * @param values the input Map to be validated
   * @param context the validation context
   * @return true if the input Map is null or contain only valid characteristics, false otherwise
   */
  @Override
  public boolean isValid(@Nullable Map<String, Integer> values,
                         ConstraintValidatorContext context) {
    if (values == null) {
      return true;
    }
    Set<String> attributes = values.keySet();
    for (String attribute : attributes) {
      if (!validCharacteristics.contains(attribute)) {
        return false;
      }
    }
    return true;
  }
}


