package cz.muni.pa165.driver.rest;

import cz.muni.pa165.common_library.dtos.DriverAddDto;
import cz.muni.pa165.common_library.dtos.DriverInsightDto;
import cz.muni.pa165.common_library.dtos.DriverResponseDto;
import cz.muni.pa165.common_library.dtos.DriverUpdateDto;
import cz.muni.pa165.driver.service.DriverService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import jakarta.validation.Valid;
import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * Rest controller for driver.
 */
@RestController
@RequestMapping(path = "/driver")
@Validated
public class DriverController {

  private final DriverService driverService;

  @Autowired
  public DriverController(DriverService driverService) {
    this.driverService = driverService;
  }

  @Operation(summary = "Add a specific driver to the team")
  @PostMapping(path = "/add", consumes = MediaType.APPLICATION_JSON_VALUE,
      produces = MediaType.APPLICATION_JSON_VALUE)
  public ResponseEntity<DriverResponseDto> addDriver(
      @Valid
      @RequestBody
      @Schema(example = "{\"name\":\"Fernando\", "
          + "\"surname\": \"Alonso\", "
          + "\"nationality\": \"Spanish\", "
          + "\"characteristics\" : {\"Experience\" : 1,"
          + "                               \"Aggressiveness\" : 1,"
          + "                               \"Consistency\" : 1,"
          + "                               \"Racecraft\" : 1 }}") DriverAddDto driverAddDto) {
    return ResponseEntity.ok(driverService.addDriver(driverAddDto));
  }

  @Operation(summary = "Updates attributes of driver specified by his id")
  @PutMapping(path = "/update/id={id}", consumes = MediaType.APPLICATION_JSON_VALUE,
      produces = MediaType.APPLICATION_JSON_VALUE)
  public ResponseEntity<DriverResponseDto> updateDriverById(
      @PathVariable("id") Long id, @Valid @RequestBody DriverUpdateDto driverUpdateDto) {
    return ResponseEntity.ok(driverService.updateDriverById(id, driverUpdateDto));
  }

  @Operation(summary = "Dismiss a driver specified by his id from the team")
  @DeleteMapping(path = "/remove/id={id}", produces = MediaType.APPLICATION_JSON_VALUE)
  public ResponseEntity<DriverResponseDto> removeDriverById(@PathVariable("id") Long id) {
    return ResponseEntity.ok(driverService.removeDriverById(id));
  }


  @Operation(summary = "Get all the signed driver of the team")
  @GetMapping(path = "/get/all", produces = MediaType.APPLICATION_JSON_VALUE)
  public ResponseEntity<List<DriverInsightDto>> getAllDrivers() {
    return ResponseEntity.ok(driverService.getAllDrivers());
  }


  @Operation(summary = "Get a signed driver specified by his id")
  @GetMapping(path = "/get/id={id}", produces = MediaType.APPLICATION_JSON_VALUE)
  public ResponseEntity<DriverInsightDto> getDriverById(@PathVariable("id") Long id) {
    return ResponseEntity.ok(driverService.getDriverById(id));
  }
}
