package cz.muni.pa165.driver.service;

import cz.muni.pa165.common_library.dtos.DriverAddDto;
import cz.muni.pa165.common_library.dtos.DriverInsightDto;
import cz.muni.pa165.common_library.dtos.DriverResponseDto;
import cz.muni.pa165.common_library.dtos.DriverUpdateDto;
import cz.muni.pa165.common_library.exception.ResourceNotFoundException;
import cz.muni.pa165.driver.data.model.Driver;
import cz.muni.pa165.driver.data.repository.DriverRepository;
import cz.muni.pa165.driver.mapper.DriverMapper;
import java.util.List;
import java.util.stream.Collectors;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * Implementation of driver service.
 */
@Service
public class DriverServiceImpl implements DriverService {

  private final DriverRepository driverRepository;
  private final DriverMapper driverMapper;

  @Autowired
  DriverServiceImpl(DriverRepository driverRepository, DriverMapper driverMapper) {
    this.driverRepository = driverRepository;
    this.driverMapper = driverMapper;
  }

  /**
   * Adds given driver to database.
   *
   * @param driverAddDto driver dto object
   * @return dto of added driver
   */
  public DriverResponseDto addDriver(DriverAddDto driverAddDto) {
    var driver = driverMapper.convertToDriver(driverAddDto);
    return driverMapper.convertToResponseDto(driverRepository.save(driver));
  }

  /**
   * Updates driver with given id by given data.
   *
   * @param id              driver id
   * @param driverUpdateDto data to be updated
   * @return dto of updated driver
   */
  public DriverResponseDto updateDriverById(Long id, DriverUpdateDto driverUpdateDto) {
    var driver = driverRepository.findById(id)
        .orElseThrow(() -> new ResourceNotFoundException("Driver with given id not found."));
    updateDriverAttributes(driver, driverUpdateDto);
    return driverMapper.convertToResponseDto(driverRepository.save(driver));
  }

  /**
   * Updates given driver attributes that were specified to be updated.
   *
   * @param driver          existing driver
   * @param driverUpdateDto driver update dto
   */
  private static void updateDriverAttributes(Driver driver, DriverUpdateDto driverUpdateDto) {
    if (driverUpdateDto.name() != null) {
      driver.setName(driverUpdateDto.name());
    }
    if (driverUpdateDto.surname() != null) {
      driver.setSurname(driverUpdateDto.surname());
    }
    if (driverUpdateDto.nationality() != null) {
      driver.setNationality(driverUpdateDto.nationality());
    }
    if (driverUpdateDto.characteristics() != null) {
      driver.getCharacteristics().putAll(driverUpdateDto.characteristics());
    }
  }

  /**
   * Removes driver with given id.
   *
   * @param id driver id
   * @return dto of removed driver
   */
  public DriverResponseDto removeDriverById(Long id) {
    var found = driverRepository.findById(id)
        .orElseThrow(() -> new ResourceNotFoundException(
            "Driver with specified id was not found."));
    driverRepository.delete(found);
    return driverMapper.convertToResponseDto(found);
  }

  /**
   * Gets all stored drivers.
   *
   * @return all stored drivers dto list
   */
  public List<DriverInsightDto> getAllDrivers() {
    return driverRepository.findAll()
        .stream()
        .map(driverMapper::convertToInsightDto)
        .collect(Collectors.toList());
  }

  /**
   * Gets driver with given id.
   *
   * @param id driver id
   * @return found driver dto if successful
   */
  public DriverInsightDto getDriverById(Long id) {
    return driverMapper.convertToInsightDto(driverRepository.findById(id)
        .orElseThrow(() -> new ResourceNotFoundException(
            "Driver with specified id was not found.")));
  }
}

