package cz.muni.pa165.race.service;

import cz.muni.pa165.car.data.model.Car;
import cz.muni.pa165.common_library.dtos.RaceDriverCarDto;
import cz.muni.pa165.common_library.dtos.RaceDto;
import cz.muni.pa165.common_library.exception.BadRequestException;
import cz.muni.pa165.common_library.exception.DatabaseException;
import cz.muni.pa165.driver.data.model.Driver;
import cz.muni.pa165.race.data.model.Race;
import cz.muni.pa165.race.data.repository.CarRepository;
import cz.muni.pa165.race.data.repository.DriverRepository;
import cz.muni.pa165.race.data.repository.RaceRepository;
import java.util.List;
import java.util.Objects;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

/**
 * Service for managing races.
 */
// TODO chybnú hlášku spraviť konštantu
@Service
public class RaceService {

  RaceRepository raceRepository;
  CarRepository carRepository;
  DriverRepository driverRepository;

  @Autowired
  RestTemplate client;

  RaceService(RaceRepository raceRepository, CarRepository carRepository,
              DriverRepository driverRepository) {
    this.raceRepository = raceRepository;
    this.carRepository = carRepository;
    this.driverRepository = driverRepository;
  }

  /**
   * Calls repository to insert a race into the database.
   *
   * @param raceDto race to insert.
   * @return inserted race.
   */
  public RaceDto postRace(RaceDto raceDto) {
    raceDto.setId(null);
    return convertRace(raceRepository.save(convertRaceDto(raceDto)));
  }

  /**
   * Deletes the race from repository by id.
   *
   * @param raceId race id
   */
  public String deleteRace(Long raceId) {
    raceRepository.deleteById(raceId);
    return "Race with id: " + raceId + "was succesfully deleted";
  }

  /**
   * Assigns driver one.
   */
  public RaceDto assignDriverOne(Long driverId, Long raceId, Long carId) {
    var driver = getDriver(driverId);
    var car = getCar(carId);
    var race = raceRepository.findById(raceId)
        .orElseThrow(() -> new DatabaseException("Race not found"));

    if (race.getDriver2() != null && Objects.equals(race.getDriver2().getDriverId(), driverId)) {
      throw new BadRequestException("Driver already assigned to the race as driver two");
    }

    if (race.getDriver2() != null && Objects.equals(race.getDriver2().getCarId(), carId)) {
      throw new BadRequestException("Car is already assigned to the race for driver two");
    }
    race.getDriver1().setDriverId(driver.getId());
    race.getDriver1().setCarId(car.getId());
    return convertRace(raceRepository.save(race));
  }

  /**
   * Assigns driver two.
   */
  public RaceDto assignDriverTwo(Long driverId, Long raceId, Long carId) {
    var driver = getDriver(driverId);
    var car = getCar(carId);
    var race = raceRepository.findById(raceId)
        .orElseThrow(() -> new DatabaseException("Race not found"));

    if (race.getDriver1() != null && Objects.equals(race.getDriver1().getDriverId(), driverId)) {
      throw new BadRequestException("Driver already assigned to the race as driver one");
    }
    if (race.getDriver1() != null && Objects.equals(race.getDriver1().getCarId(), carId)) {
      throw new BadRequestException("Car is already assigned to the race for driver two");
    }

    race.getDriver2().setDriverId(driver.getId());
    race.getDriver2().setCarId(car.getId());
    return convertRace(raceRepository.save(race));
  }

  /**
   * Finds race by id.
   *
   * @param raceId id of race.
   * @return found race.
   */
  public RaceDto findRaceById(Long raceId) {
    return convertRace(raceRepository.findById(raceId).orElseThrow(
        () -> new DatabaseException("Something went wrong when"
            + "finding the driver with id: " + raceId + " in the database.")));
  }

  /**
   * Find all races.
   *
   * @return found races.
   */
  public List<RaceDto> findRaces() {
    return raceRepository.findAll().stream().map(this::convertRace).toList();
  }

  private Race convertRaceDto(RaceDto raceDto) {
    var race = Race.builder()
        .id(raceDto.getId())
        .build();

    var raceInfo = Race.RaceInfo
        .builder()
        .id(raceDto.getId())
        .name(raceDto.getRaceInfo().getName())
        .location(raceDto.getRaceInfo().getLocation())
        .prizePool(raceDto.getRaceInfo().getPrizePool())
        .build();
    race.setRaceInfo(raceInfo);

    if (raceDto.getDriverOne() != null) {
      race.setDriver1(Race.RaceDriverinfo
          .builder()
          .driverId(raceDto.getDriverOne().getDriverId())
          .carId(raceDto.getDriverOne().getCarId())
          .build());
    } else {
      race.setDriver1(Race.RaceDriverinfo.builder().build());
    }

    if (raceDto.getDriverTwo() != null) {
      race.setDriver2(Race.RaceDriverinfo
          .builder()
          .driverId(raceDto.getDriverTwo().getDriverId())
          .carId(raceDto.getDriverTwo().getCarId())
          .build());
    } else {
      race.setDriver2(Race.RaceDriverinfo.builder().build());
    }

    return race;
  }

  private RaceDto convertRace(Race race) {
    var raceDto = RaceDto.builder()
        .id(race.getId())
        .raceInfo(RaceDto.RaceInfo.builder()
            .name(race.getRaceInfo().getName())
            .location(race.getRaceInfo().getLocation())
            .prizePool(race.getRaceInfo().getPrizePool())
            .build())
        .build();
    if (race.getDriver1() != null) {
      raceDto.setDriverOne(RaceDriverCarDto.builder()
          .carId(race.getDriver1().getCarId())
          .driverId(race.getDriver1().getDriverId())
          .build());
    }
    if (race.getDriver2() != null) {
      raceDto.setDriverTwo(RaceDriverCarDto.builder()
          .carId(race.getDriver2().getCarId())
          .driverId(race.getDriver2().getCarId())
          .build());
    }
    return raceDto;
  }

  private Driver getDriver(Long driverId) {
    var response =
        client.getForEntity("http://localhost:8083/driver/get/id=" + driverId, Driver.class);
    return response.getBody();
  }

  private Car getCar(Long carId) {
    var response = client.getForEntity("http://localhost:8082/car/id=" + carId, Car.class);
    return response.getBody();
  }
}
